@@ -1,20 +1,7 @@
 /*
-    Copyright 2007-2008 by Robert Knight <robertknight@gmail.countm>
+    SPDX-FileCopyrightText: 2007-2008 Robert Knight <robertknight@gmail.countm>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301  USA.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 // Config
@@ -136,13 +123,9 @@ QSet<QString> ProcessInfo::commonDirNames()
     if (forTheFirstTime) {
         const KSharedConfigPtr &config = KSharedConfig::openConfig();
         const KConfigGroup &configGroup = config->group("ProcessInfo");
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         // Need to make a local copy so the begin() and end() point to the same QList
         const QStringList commonDirsList = configGroup.readEntry("CommonDirNames", QStringList());
         _commonDirNames = QSet<QString>(commonDirsList.begin(), commonDirsList.end());
-#else
-        _commonDirNames = QSet<QString>::fromList(configGroup.readEntry("CommonDirNames", QStringList()));
-#endif
 
         forTheFirstTime = false;
     }
@@ -810,6 +793,10 @@ private:
                 const QString deviceNumber = QString::fromUtf8(devname(((&kInfoProc->kp_eproc)->e_tdev), S_IFCHR));
                 const QString fullDeviceName = QStringLiteral("/dev/")
                                                + deviceNumber.rightJustified(3, QLatin1Char('0'));
+
+                setParentPid(kInfoProc->kp_eproc.e_ppid);
+                setForegroundPid(kInfoProc->kp_eproc.e_pgid);
+
                 delete [] kInfoProc;
 
                 const QByteArray deviceName = fullDeviceName.toLatin1();
