@@ -32,38 +32,11 @@
 
 using namespace Konsole;
 
-static bool profileIndexLessThan(const Profile::Ptr& p1, const Profile::Ptr& p2)
-{
-    return p1->menuIndexAsInt() < p2->menuIndexAsInt();
-}
-
-static bool profileNameLessThan(const Profile::Ptr& p1, const Profile::Ptr& p2)
-{
-    // Always put the Default/fallback profile at the top
-    if (p1->isFallback()) {
-        return true;
-    } else if (p2->isFallback()) {
-        return false;
-    }
-
-    return QString::localeAwareCompare(p1->name(), p2->name()) < 0;
-}
-
 static bool stringLessThan(const QString& p1, const QString& p2)
 {
     return QString::localeAwareCompare(p1, p2) < 0;
 }
 
-static void sortByIndexProfileList(QList<Profile::Ptr>& list)
-{
-    std::stable_sort(list.begin(), list.end(), profileIndexLessThan);
-}
-
-static void sortByNameProfileList(QList<Profile::Ptr>& list)
-{
-    std::stable_sort(list.begin(), list.end(), profileNameLessThan);
-}
-
 ProfileManager::ProfileManager()
     : _profiles(QSet<Profile::Ptr>())
     , _defaultProfile(nullptr)
@@ -247,41 +220,17 @@ void ProfileManager::loadAllProfiles()
 
 void ProfileManager::sortProfiles(QList<Profile::Ptr>& list)
 {
-    QList<Profile::Ptr> lackingIndices;
-    QList<Profile::Ptr> havingIndices;
-
-    for (const auto & i : list) {
-        if (i->menuIndexAsInt() == 0) {
-            lackingIndices.append(i);
-        } else {
-            havingIndices.append(i);
+    // Sort alphabetically
+    std::stable_sort(list.begin(), list.end(), [](const Profile::Ptr &p1, const Profile::Ptr &p2) {
+        // Always put the Default/fallback profile at the top
+        if (p1->isFallback()) {
+            return true;
+        } else if (p2->isFallback()) {
+            return false;
         }
-    }
 
-    // sort by index
-    sortByIndexProfileList(havingIndices);
-
-    // sort alphabetically those w/o an index
-    sortByNameProfileList(lackingIndices);
-
-    // Put those with indices in sequential order w/o any gaps
-    int i = 0;
-    for (i = 0; i < havingIndices.size(); ++i) {
-        Profile::Ptr tempProfile = havingIndices.at(i);
-        tempProfile->setProperty(Profile::MenuIndex, QString::number(i + 1));
-        havingIndices.replace(i, tempProfile);
-    }
-    // Put those w/o indices in sequential order
-    for (int j = 0; j < lackingIndices.size(); ++j) {
-        Profile::Ptr tempProfile = lackingIndices.at(j);
-        tempProfile->setProperty(Profile::MenuIndex, QString::number(j + 1 + i));
-        lackingIndices.replace(j, tempProfile);
-    }
-
-    // combine the 2 list: first those who had indices
-    list.clear();
-    list.append(havingIndices);
-    list.append(lackingIndices);
+        return QString::localeAwareCompare(p1->name(), p2->name()) < 0;
+    });
 }
 
 void ProfileManager::saveSettings()
@@ -302,7 +251,7 @@ QList<Profile::Ptr> ProfileManager::allProfiles()
     loadAllProfiles();
 
     auto loadedProfiles = _profiles.values();
-    sortByNameProfileList(loadedProfiles);
+    sortProfiles(loadedProfiles);
 
     return loadedProfiles;
 }
@@ -321,6 +270,19 @@ Profile::Ptr ProfileManager::fallbackProfile() const
     return _fallbackProfile;
 }
 
+QString ProfileManager::generateUniqueName() const
+{
+    const QStringList existingProfileNames = availableProfileNames();
+    int nameSuffix = 1;
+    QString uniqueProfileName;
+    do {
+        uniqueProfileName = QStringLiteral("Profile ") + QString::number(nameSuffix);
+        ++nameSuffix;
+    } while (existingProfileNames.contains(uniqueProfileName));
+
+    return uniqueProfileName;
+}
+
 QString ProfileManager::saveProfile(const Profile::Ptr &profile)
 {
     ProfileWriter writer;
@@ -340,34 +302,13 @@ void ProfileManager::changeProfile(Profile::Ptr profile,
 {
     Q_ASSERT(profile);
 
-    const bool isFallback = profile->isFallback();
     const QString origPath = profile->path();
 
-    const QStringList existingProfileNames = availableProfileNames();
-    // Generate a unique profile name
-    int nameSuffix = 1;
-    QString uniqueProfileName;
-    do {
-        uniqueProfileName = QStringLiteral("Profile ") + QString::number(nameSuffix);
-        ++nameSuffix;
-    } while (existingProfileNames.contains(uniqueProfileName));
-
-
+    const QString uniqueProfileName = generateUniqueName();
 
     // Don't save a profile with an empty name on disk
     persistent = persistent && !profile->name().isEmpty();
 
-    // If we are asked to store the fallback profile (which has an
-    // invalid path by design), we reset the path to an empty string
-    // which will make the profile writer automatically generate a
-    // proper path.
-    if (persistent && isFallback) {
-        profile->setProperty(Profile::UntranslatedName, uniqueProfileName);
-        profile->setProperty(Profile::Name, uniqueProfileName);
-        profile->setProperty(Profile::MenuIndex, QStringLiteral("0"));
-        profile->setHidden(false);
-    }
-
     bool messageShown = false;
     // Insert the changes into the existing Profile instance
     for (auto it = propertyMap.cbegin(); it != propertyMap.cend(); ++it) {
@@ -420,7 +361,7 @@ void ProfileManager::changeProfile(Profile::Ptr profile,
         // when creating a new profile, this works around a bug where
         // the newly created profile appears twice in the ProfileSettings
         // dialog
-        if (!isFallback && !origPath.isEmpty() && profile->path() != origPath) {
+        if (!origPath.isEmpty() && profile->path() != origPath) {
             // this is needed to include the old profile too
             _loadedAllProfiles = false;
             const QList<Profile::Ptr> availableProfiles = ProfileManager::instance()->allProfiles();
@@ -437,22 +378,8 @@ void ProfileManager::changeProfile(Profile::Ptr profile,
         }
     }
 
-    if (isFallback) {
-        // addProfile has to be called after a path has been set for
-        // "profile", otherwise the newly saved profile won't show up
-        // in the ProfileSettings dialog in the current Konsole window
-        addProfile(profile);
-        setDefaultProfile(profile);
-
-        // Since the profile object pointed to by _fallbackProfile has
-        // been given a name above, now init a fallback profile again.
-        // This way there is always a "Default" profile available in the
-        // context menu.
-        initFallbackProfile();
-    } else {
-        // notify the world about the change
-        Q_EMIT profileChanged(profile);
-    }
+    // Notify the world about the change
+    Q_EMIT profileChanged(profile);
 }
 
 void ProfileManager::addProfile(const Profile::Ptr &profile)
