@@ -1,24 +1,9 @@
 /*
-    This file is part of Konsole
+    SPDX-FileCopyrightText: 2006-2008 Robert Knight <robertknight@gmail.com>
+    SPDX-FileCopyrightText: 1997, 1998 Lars Doelle <lars.doelle@on-line.de>
+    SPDX-FileCopyrightText: 2009 Thomas Dreibholz <dreibh@iem.uni-due.de>
 
-    Copyright 2006-2008 by Robert Knight <robertknight@gmail.com>
-    Copyright 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
-    Copyright 2009 by Thomas Dreibholz <dreibh@iem.uni-due.de>
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301  USA.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 // Own
@@ -43,6 +28,7 @@
 #include <KProcess>
 #include <KConfigGroup>
 #include <KIO/DesktopExecParser>
+#include <kcoreaddons_version.h>
 
 // Konsole
 #include <sessionadaptor.h>
@@ -61,7 +47,13 @@
 #include "SessionManager.h"
 
 #include "terminalDisplay/TerminalDisplay.h"
-#include "terminalDisplay/TerminalScrollBar.hpp"
+#include "terminalDisplay/TerminalScrollBar.h"
+
+// Linux
+#ifdef HAVE_GETPWUID
+#include <sys/types.h>
+#include <pwd.h>
+#endif
 
 using namespace Konsole;
 
@@ -126,7 +118,7 @@ Session::Session(QObject* parent) :
 
     connect(_emulation, &Konsole::Emulation::sessionAttributeChanged, this, &Konsole::Session::setSessionAttribute);
     connect(_emulation, &Konsole::Emulation::bell, this, [this]() {
-        emit bellRequest(i18n("Bell in session '%1'", _nameTitle));
+        Q_EMIT bellRequest(i18n("Bell in session '%1'", _nameTitle));
         this->setPendingNotification(Notification::Bell);
     });
     connect(_emulation, &Konsole::Emulation::zmodemDownloadDetected, this, &Konsole::Session::fireZModemDownloadDetected);
@@ -191,10 +183,12 @@ void Session::openTeletype(int fd, bool runShell)
             this, &Konsole::Session::done);
 
     // emulator size
-    // Use a direct connection to ensure that the window size is set before it runs
-    connect(_emulation, &Konsole::Emulation::imageSizeChanged, this, &Konsole::Session::updateWindowSize, Qt::DirectConnection);
+    connect(_emulation, &Konsole::Emulation::imageSizeChanged, this, &Konsole::Session::updateWindowSize);
     if (fd < 0 || runShell) {
-        connect(_emulation, &Konsole::Emulation::imageSizeInitialized, this, &Konsole::Session::run);
+        // Using a queued connection guarantees that starting the session
+        // is delayed until all (both) image size updates at startup have
+        // been processed. See #203185 and #412598.
+        connect(_emulation, &Konsole::Emulation::imageSizeInitialized, this, &Konsole::Session::run, Qt::QueuedConnection);
     } else {
         // run needs to be disconnected, as it may be already connected by the constructor
         disconnect(_emulation, &Konsole::Emulation::imageSizeInitialized, this, &Konsole::Session::run);
@@ -255,7 +249,7 @@ void Session::setCodec(QTextCodec* codec)
 
     emulation()->setCodec(codec);
 
-    emit sessionCodecChanged(codec);
+    Q_EMIT sessionCodecChanged(codec);
 }
 
 bool Session::setCodec(const QByteArray& name)
@@ -310,7 +304,7 @@ void Session::updateWorkingDirectory()
     const QString currentDir = _sessionProcessInfo->validCurrentDir();
     if (currentDir != _currentWorkingDir) {
         _currentWorkingDir = currentDir;
-        emit currentDirectoryChanged(_currentWorkingDir);
+        Q_EMIT currentDirectoryChanged(_currentWorkingDir);
     }
 }
 
@@ -444,7 +438,7 @@ void Session::run()
 {
     // FIXME: run() is called twice in some instances
     if (isRunning()) {
-        qCDebug(KonsoleDebug) << "Attempted to re-run an already running session (" << _shellProcess->pid() << ")";
+        qCDebug(KonsoleDebug) << "Attempted to re-run an already running session (" << processId() << ")";
         return;
     }
 
@@ -459,26 +453,30 @@ void Session::run()
         _uniqueIdentifier = QUuid::createUuid();
     }
 
-    const int CHOICE_COUNT = 3;
-    // if '_program' is empty , fall back to default shell. If that is not set
-    // then fall back to /bin/sh
-    QString programs[CHOICE_COUNT] = {_program, QString::fromUtf8(qgetenv("SHELL")), QStringLiteral("/bin/sh")};
+    QStringList programs = {_program, QString::fromUtf8(qgetenv("SHELL")), QStringLiteral("/bin/sh")};
+
+#ifdef HAVE_GETPWUID
+    auto pw = getpwuid(getuid());
+    // pw may be NULL
+    if (pw != nullptr) {
+        programs.insert(1, QString::fromLocal8Bit(pw->pw_shell));
+    }
+    // pw: Do not pass the returned pointer to free.
+#endif
+
     QString exec;
-    int choice = 0;
-    while (choice < CHOICE_COUNT) {
-        exec = checkProgram(programs[choice]);
-        if (exec.isEmpty()) {
-            choice++;
-        } else {
+    for (const auto &choice : programs) {
+        exec = checkProgram(choice);
+        if (!exec.isEmpty()) {
             break;
         }
     }
 
     // if a program was specified via setProgram(), but it couldn't be found, print a warning
-    if (choice != 0 && choice < CHOICE_COUNT && !_program.isEmpty()) {
+    if (exec != checkProgram(_program)) {
         terminalWarning(i18n("Could not find '%1', starting '%2' instead.  Please check your profile settings.", _program, exec));
         // if none of the choices are available, print a warning
-    } else if (choice == CHOICE_COUNT) {
+    } else if (programs.last() == checkProgram(exec)) {
         terminalWarning(i18n("Could not find an interactive shell to start."));
         return;
     }
@@ -524,7 +522,7 @@ void Session::run()
 
     _shellProcess->setWriteable(false);  // We are reachable via kwrited.
 
-    emit started();
+    Q_EMIT started();
 }
 
 void Session::setSessionAttribute(int what, const QString& caption)
@@ -552,9 +550,9 @@ void Session::setSessionAttribute(int what, const QString& caption)
         QColor color = QColor(colorString);
         if (color.isValid()) {
             if (what == TextColor) {
-                emit changeForegroundColorRequest(color);
+                Q_EMIT changeForegroundColorRequest(color);
             } else {
-                emit changeBackgroundColorRequest(color);
+                Q_EMIT changeBackgroundColorRequest(color);
             }
         }
     }
@@ -581,17 +579,17 @@ void Session::setSessionAttribute(int what, const QString& caption)
 
     if (what == CurrentDirectory) {
         _reportedWorkingUrl = QUrl::fromUserInput(caption);
-        emit currentDirectoryChanged(currentWorkingDirectory());
+        Q_EMIT currentDirectoryChanged(currentWorkingDirectory());
         modified = true;
     }
 
     if (what == ProfileChange) {
-        emit profileChangeCommandReceived(caption);
+        Q_EMIT profileChangeCommandReceived(caption);
         return;
     }
 
     if (modified) {
-        emit sessionAttributeChanged();
+        Q_EMIT sessionAttributeChanged();
     }
 }
 
@@ -701,7 +699,7 @@ void Session::changeTabTextColor(int i)
 void Session::onPrimaryScreenInUse(bool use)
 {
     _isPrimaryScreen = use;
-    emit primaryScreenInUse(use);
+    Q_EMIT primaryScreenInUse(use);
 }
 
 bool Session::isPrimaryScreen()
@@ -714,11 +712,11 @@ void Session::sessionAttributeRequest(int id, uint terminator)
     switch (id) {
         case TextColor:
             // Get 'TerminalDisplay' (_view) foreground color
-            emit getForegroundColor(terminator);
+            Q_EMIT getForegroundColor(terminator);
             break;
         case BackgroundColor:
             // Get 'TerminalDisplay' (_view) background color
-            emit getBackgroundColor(terminator);
+            Q_EMIT getBackgroundColor(terminator);
             break;
     }
 }
@@ -792,6 +790,8 @@ void Session::sendSignal(int signal)
 
     if (ok) {
         ::kill(pid, signal);
+    } else {
+        qWarning()<<"foreground process id not set, unable to send signal "<<signal;
     }
 }
 
@@ -825,11 +825,11 @@ void Session::reportBackgroundColor(const QColor& c, uint terminator)
 
 bool Session::kill(int signal)
 {
-    if (_shellProcess->pid() <= 0) {
+    if (processId() <= 0) {
         return false;
     }
 
-    int result = ::kill(_shellProcess->pid(), signal);
+    int result = ::kill(processId(), signal);
 
     if (result == 0) {
         return _shellProcess->waitForFinished(1000);
@@ -862,7 +862,7 @@ bool Session::closeInNormalWay()
     // 3). the user closes the tab explicitly
     //
     if (!isRunning()) {
-        emit finished();
+        Q_EMIT finished();
         return true;
     }
 
@@ -883,7 +883,7 @@ bool Session::closeInNormalWay()
     if (kill(SIGHUP)) {
         return true;
     } else {
-        qWarning() << "Process " << _shellProcess->pid() << " did not die with SIGHUP";
+        qWarning() << "Process " << processId() << " did not die with SIGHUP";
         _shellProcess->closePty();
         return (_shellProcess->waitForFinished(1000));
     }
@@ -897,7 +897,7 @@ bool Session::closeInForceWay()
     if (kill(SIGKILL)) {
         return true;
     } else {
-        qWarning() << "Process " << _shellProcess->pid() << " did not die with SIGKILL";
+        qWarning() << "Process " << processId() << " did not die with SIGKILL";
         return false;
     }
 }
@@ -963,12 +963,12 @@ void Session::done(int exitCode, QProcess::ExitStatus exitStatus)
 
     if (!_autoClose) {
         _userTitle = i18nc("@info:shell This session is done", "Finished");
-        emit sessionAttributeChanged();
+        Q_EMIT sessionAttributeChanged();
         return;
     }
 
     if (_closePerUserRequest) {
-        emit finished();
+        Q_EMIT finished();
         return;
     }
 
@@ -992,7 +992,7 @@ void Session::done(int exitCode, QProcess::ExitStatus exitStatus)
         message = i18n("Program '%1' crashed.", _program);
         terminalWarning(message);
     } else {
-        emit finished();
+        Q_EMIT finished();
     }
 }
 
@@ -1044,7 +1044,7 @@ void Session::setTitle(TitleRole role , const QString& newTitle)
             _displayTitle = newTitle;
         }
 
-        emit sessionAttributeChanged();
+        Q_EMIT sessionAttributeChanged();
     }
 }
 
@@ -1246,7 +1246,7 @@ void Session::setIconName(const QString& iconName)
 {
     if (iconName != _iconName) {
         _iconName = iconName;
-        emit sessionAttributeChanged();
+        Q_EMIT sessionAttributeChanged();
     }
 }
 
@@ -1365,7 +1365,7 @@ void Session::setFlowControlEnabled(bool enabled)
         _shellProcess->setFlowControlEnabled(_flowControlEnabled);
     }
 
-    emit flowControlEnabledChanged(enabled);
+    Q_EMIT flowControlEnabledChanged(enabled);
 }
 bool Session::flowControlEnabled() const
 {
@@ -1516,7 +1516,7 @@ void Session::setSize(const QSize& size)
         return;
     }
 
-    emit resizeRequest(size);
+    Q_EMIT resizeRequest(size);
 }
 
 QSize Session::preferredSize() const
@@ -1531,7 +1531,7 @@ void Session::setPreferredSize(const QSize& size)
 
 int Session::processId() const
 {
-    return _shellProcess->pid();
+    return _shellProcess->processId();
 }
 
 void Session::setTitle(int role , const QString& title)
@@ -1649,7 +1649,11 @@ int Session::foregroundProcessId()
 bool Session::isForegroundProcessActive()
 {
     // foreground process info is always updated after this
-    return (_shellProcess->pid() != _shellProcess->foregroundProcessGroup());
+
+    // This check is wrong when Konsole is started with '-e cmd'
+    // as there will only be one process.
+    // See BKO 134581 for no popup when closing session
+    return (processId() != _shellProcess->foregroundProcessGroup());
 }
 
 QString Session::foregroundProcessName()
@@ -1726,7 +1730,7 @@ void Session::setPendingNotification(Session::Notification notification, bool en
 {
     if(enable != _activeNotifications.testFlag(notification)) {
         _activeNotifications.setFlag(notification, enable);
-        emit notificationsChanged(notification, enable);
+        Q_EMIT notificationsChanged(notification, enable);
     }
 }
 
@@ -1768,14 +1772,14 @@ void Session::setReadOnly(bool readOnly)
 
         // Needed to update the tab icons and all
         // attached views.
-        emit readOnlyChanged();
+        Q_EMIT readOnlyChanged();
     }
 }
 
 void Session::setColor(const QColor &color)
 {
     _tabColor = color;
-    emit sessionAttributeChanged();
+    Q_EMIT sessionAttributeChanged();
 }
 
 QColor Session::color() const
