@@ -1,21 +1,8 @@
 /*
-    Copyright 2007-2008 by Robert Knight <robertknight@gmail.com>
-    Copyright 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
+    SPDX-FileCopyrightText: 2007-2008 Robert Knight <robertknight@gmail.com>
+    SPDX-FileCopyrightText: 1997, 1998 Lars Doelle <lars.doelle@on-line.de>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301  USA.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #ifndef TERMINALDISPLAY_H
@@ -51,6 +38,8 @@ class KMessageWidget;
 namespace Konsole {
 class TerminalPainter;
 class TerminalScrollBar;
+class TerminalColor;
+class TerminalFont;
 
 class KonsolePrintManager;
 
@@ -84,10 +73,6 @@ public:
 
     void applyProfile(const QExplicitlySharedDataPointer<Profile>& profile);
 
-    /** Returns the terminal color palette used by the display. */
-    const ColorEntry *colorTable() const;
-    /** Sets the terminal color palette used by the display. */
-    void setColorTable(const ColorEntry table[]);
     /**
      * Sets the seed used to generate random colors for the display
      * (in color schemes that support them).
@@ -146,9 +131,6 @@ public:
     /** Specifies whether or not text can blink. */
     void setBlinkingTextEnabled(bool blink);
 
-    void setLineSpacing(uint);
-    uint lineSpacing() const;
-
     void setSessionController(SessionController *controller);
     SessionController *sessionController();
 
@@ -176,36 +158,6 @@ public:
     */
     void resetCursorStyle();
 
-    /**
-     * Sets the color used to draw the keyboard cursor.
-     *
-     * The keyboard cursor defaults to using the foreground color of the character
-     * underneath it.
-     *
-     * @param color By default, the widget uses the color of the
-     * character under the cursor to draw the cursor, and inverts the
-     * color of that character to make sure it is still readable. If @p
-     * color is a valid QColor, the widget uses that color to draw the
-     * cursor. If @p color is not an valid QColor, the widget falls back
-     * to the default behavior.
-     */
-    void setKeyboardCursorColor(const QColor &color);
-
-    /**
-     * Sets the color used to draw the character underneath the keyboard cursor.
-     *
-     * The keyboard cursor defaults to using the background color of the
-     * terminal cell to draw the character at the cursor position.
-     *
-     * @param color By default, the widget uses the color of the
-     * character under the cursor to draw the cursor, and inverts the
-     * color of that character to make sure it is still readable. If @p
-     * color is a valid QColor, the widget uses that color to draw the
-     * character underneath the cursor. If @p color is not an valid QColor,
-     * the widget falls back to the default behavior.
-     */
-    void setKeyboardCursorTextColor(const QColor &color);
-
     /**
      * Returns the number of lines of text which can be displayed in the widget.
      *
@@ -229,24 +181,13 @@ public:
         return _columns;
     }
 
-    /**
-     * Returns the height of the characters in the font used to draw the text in the display.
-     */
-    int  fontHeight() const
+    int usedColumns() const
     {
-        return _fontHeight;
-    }
-
-    /**
-     * Returns the width of the characters in the display.
-     * This assumes the use of a fixed-width font.
-     */
-    int  fontWidth() const
-    {
-        return _fontWidth;
+        return _usedColumns;
     }
 
     void setSize(int columns, int lines);
+    void propagateSize();
 
     // reimplemented
     QSize sizeHint() const override;
@@ -280,33 +221,10 @@ public:
      */
     int bellMode() const;
 
-    /** Play a visual bell for prompt or warning. */
-    void visualBell();
-
-    /** Returns the font used to draw characters in the display */
-    QFont getVTFont()
-    {
-        return font();
-    }
-
     TerminalHeaderBar *headerBar() const
     {
         return _headerBar;
     }
-    /**
-     * Sets the font used to draw the display.  Has no effect if @p font
-     * is larger than the size of the display itself.
-     */
-    void setVTFont(const QFont &f);
-
-    /** Increases the font size */
-    void increaseFontSize();
-
-    /** Decreases the font size */
-    void decreaseFontSize();
-
-    /** Reset the font size */
-    void resetFontSize();
 
     void resetCursor();
 
@@ -336,18 +254,6 @@ public:
      */
     void selectAll();
 
-    /**
-     * Gets the foreground of the display
-     * @see setForegroundColor(), setColorTable(), setBackgroundColor()
-     */
-    QColor getForegroundColor() const;
-
-    /**
-     * Gets the background of the display
-     * @see setBackgroundColor(), setColorTable(), setForegroundColor()
-     */
-    QColor getBackgroundColor() const;
-
     bool bracketedPasteMode() const;
 
     /**
@@ -374,19 +280,62 @@ public:
     // maps a point on the widget to the position ( ie. line and column )
     // of the character at that point. When the edge is true, it maps to
     // a character which left edge is closest to the point.
-    void getCharacterPosition(const QPoint &widgetPoint, int &line, int &column, bool edge) const;
+    QPair<int, int> getCharacterPosition(const QPoint &widgetPoint, bool edge) const;
 
     // toggle the header bar Minimize/Maximize button.
     void setExpandedMode(bool expand);
 
-    friend class TerminalPainter;
-    friend class TerminalScrollBar;
-
-    TerminalScrollBar *scrollBar()
+    TerminalScrollBar *scrollBar() const
     {
         return _scrollBar;
     }
 
+    TerminalColor *terminalColor() const
+    {
+        return _terminalColor;
+    }
+
+    TerminalFont *terminalFont() const
+    {
+        return _terminalFont;
+    }
+
+    bool cursorBlinking() const
+    {
+        return _cursorBlinking;
+    }
+
+    bool textBlinking() const
+    {
+        return _textBlinking;
+    }
+
+    Enum::CursorShapeEnum cursorShape() const
+    {
+        return _cursorShape;
+    }
+
+    bool bidiEnabled() const
+    {
+        return _bidiEnabled;
+    }
+
+    ColorSchemeWallpaper::Ptr wallpaper() const
+    {
+        return _wallpaper;
+    }
+
+    struct InputMethodData {
+        QString preeditString;
+        QRect previousPreeditRect;
+    };
+
+    // returns true if the cursor's position is on display.
+    bool isCursorOnDisplay() const;
+
+    // returns the position of the cursor in columns and lines
+    QPoint cursorPosition() const;
+
 public Q_SLOTS:
     /**
      * Scrolls current ScreenWindow
@@ -472,18 +421,6 @@ public Q_SLOTS:
      */
     void bell(const QString &message);
 
-    /**
-     * Sets the background of the display to the specified color.
-     * @see setColorTable(), getBackgroundColor(), setForegroundColor()
-     */
-    void setBackgroundColor(const QColor &color);
-
-    /**
-     * Sets the text of the display to the specified color.
-     * @see setColorTable(), setBackgroundColor(), getBackgroundColor()
-     */
-    void setForegroundColor(const QColor &color);
-
     /**
      * Sets the display's contents margins.
      */
@@ -514,8 +451,12 @@ public Q_SLOTS:
     void printScreen();
     Character getCursorCharacter(int column, int line);
 
+    int loc(int x, int y) const;
+
 Q_SIGNALS:
     void requestToggleExpansion();
+    void requestMoveToNewTab(TerminalDisplay* display);
+
     /**
      * Emitted when the user presses a key whilst the terminal widget has focus.
      */
@@ -529,7 +470,6 @@ Q_SIGNALS:
      * @param eventType The type of event.  0 for a mouse press / release or 1 for mouse motion
      */
     void mouseSignal(int button, int column, int line, int eventType);
-    void changedFontMetricSignal(int height, int width);
     void changedContentSizeSignal(int height, int width);
 
     /**
@@ -557,6 +497,18 @@ Q_SIGNALS:
 
     void peekPrimaryRequested(bool doPeek);
 
+    void drawContents(Character *image, QPainter &paint, const QRect &rect, bool printerFriendly, int imageSize, bool bidiEnabled,
+        QVector<LineProperty> lineProperties);
+    void drawCurrentResultRect(QPainter &painter, QRect searchResultRect);
+    
+    void highlightScrolledLines(QPainter& painter, bool isTimerActive, QRect rect);
+    QRegion highlightScrolledLinesRegion(bool nothingChanged, TerminalScrollBar *scrollBar);
+    
+    void drawBackground(QPainter &painter, const QRect &rect, const QColor &backgroundColor, bool useOpacitySetting);
+    void drawCharacters(QPainter &painter, const QRect &rect, const QString &text,
+                            const Character *style, const QColor &characterColor);
+    void drawInputMethodPreeditString(QPainter &painter, const QRect &rect, TerminalDisplay::InputMethodData &inputMethodData, Character *image);
+
 protected:
     // events
     bool event(QEvent *event) override;
@@ -577,7 +529,6 @@ protected:
     void wheelEvent(QWheelEvent *ev) override;
     bool focusNextPrevChild(bool next) override;
 
-    void fontChange(const QFont &);
     void extendSelection(const QPoint &position);
 
     // drag and drop
@@ -611,8 +562,6 @@ protected:
     void inputMethodEvent(QInputMethodEvent *event) override;
     QVariant inputMethodQuery(Qt::InputMethodQuery query) const override;
 
-    void onColorsChanged();
-
 protected Q_SLOTS:
 
     void blinkTextEvent();
@@ -620,7 +569,6 @@ protected Q_SLOTS:
 
 private Q_SLOTS:
 
-    void swapFGBGColors();
     void viewScrolledByUser();
 
     void dismissOutputSuspendedMessage();
@@ -636,7 +584,6 @@ private:
     void showResizeNotification();
 
     void calcGeometry();
-    void propagateSize();
     void updateImageSize();
     void makeImage();
 
@@ -644,12 +591,6 @@ private:
 
     void setupHeaderVisibility();
 
-    // returns the position of the cursor in columns and lines
-    QPoint cursorPosition() const;
-
-    // returns true if the cursor's position is on display.
-    bool isCursorOnDisplay() const;
-
     // redraws the cursor
     void updateCursor();
 
@@ -670,8 +611,6 @@ private:
     // Boilerplate setup for MessageWidget
     KMessageWidget* createMessageWidget(const QString &text);
 
-    int loc(int x, int y) const;
-
     // the window onto the terminal screen which this display
     // is currently showing.
     QPointer<ScreenWindow> _screenWindow;
@@ -680,15 +619,14 @@ private:
 
     QVBoxLayout *_verticalLayout;
 
-    bool _fixedFont; // has fixed pitch
-    int _fontHeight;      // height
-    int _fontWidth;      // width
-    int _fontAscent;      // ascend
-    bool _boldIntense;   // Whether intense colors should be rendered with bold font
-
     int _lines;      // the number of lines that can be displayed in the widget
     int _columns;    // the number of columns that can be displayed in the widget
 
+    // Character line and character column as per a previous call to
+    // getCharacterPosition() in mouseMoveEvent().
+    int _prevCharacterLine;
+    int _prevCharacterColumn;
+
     int _usedLines;  // the number of lines that are actually being used, this will be less
     // than 'lines' if the character image provided with setImage() is smaller
     // than the maximum image size which can be displayed
@@ -704,7 +642,7 @@ private:
     int _imageSize;
     QVector<LineProperty> _lineProperties;
 
-    ColorEntry _colorTable[TABLE_COLORS];
+    QColor _colorTable[TABLE_COLORS];
 
     uint _randomSeed;
 
@@ -756,12 +694,8 @@ private:
     //terminal output - informing them what has happened and how to resume output
     KMessageWidget *_outputSuspendedMessageWidget;
 
-    uint _lineSpacing;
-
     QSize _size;
 
-    QRgb _blendColor;
-
     ColorScheme const* _colorScheme;
     ColorSchemeWallpaper::Ptr _wallpaper;
 
@@ -772,23 +706,8 @@ private:
 
     Enum::CursorShapeEnum _cursorShape;
 
-    // cursor color. If it is invalid (by default) then the foreground
-    // color of the character under the cursor is used
-    QColor _cursorColor;
-
-    // cursor text color. If it is invalid (by default) then the background
-    // color of the character under the cursor is used
-    QColor _cursorTextColor;
-
-    struct InputMethodData {
-        QString preeditString;
-        QRect previousPreeditRect;
-    };
     InputMethodData _inputMethodData;
 
-    bool _antialiasText;   // do we anti-alias or not
-    bool _useFontLineCharacters;
-
     //the delay in milliseconds between redrawing blinking text
     static const int TEXT_BLINK_DELAY = 500;
 
@@ -809,8 +728,6 @@ private:
     // Needed to know whether the mode really changed between update calls
     bool _readOnly;
 
-    qreal _opacity;
-
     bool _dimWhenInactive;
     int _dimValue;
 
@@ -823,15 +740,6 @@ private:
     bool _drawOverlay;
     Qt::Edge _overlayEdge;
 
-    struct {
-        bool enabled = false;
-        QRect rect;
-        int previousScrollCount = 0;
-        QTimer *timer = nullptr;
-        bool needToClear = false;
-    } _highlightScrolledLinesControl;
-    static const int HIGHLIGHT_SCROLLED_LINES_WIDTH = 3;
-
     bool _hasCompositeFocus;
     bool _displayVerticalLine;
     int _displayVerticalLineAtChar;
@@ -840,6 +748,8 @@ private:
 
     TerminalPainter *_terminalPainter;
     TerminalScrollBar *_scrollBar;
+    TerminalColor *_terminalColor;
+    TerminalFont *_terminalFont;
 
     KonsolePrintManager *_printManager;
 };
